# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2019, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
"""Utilities for .projectignore file."""

import os
import codecs

_default_projectignore = """
# This file contains a list of match patterns that instructs
# anaconda-project to exclude certain files or directories when
# building a project archive. The file format is a simplfied
# version of Git's .gitignore file format. In fact, if the
# project is hosted in a Git repository, these patterns can be
# merged into the .gitignore file and this file removed.
# See the anaconda-project documentation for more details.

# Python caching
*.pyc
*.pyd
*.pyo
__pycache__/

# Jupyter & Spyder stuff
.ipynb_checkpoints/
.Trash-*/
/.spyderproject
""".lstrip()


def add_projectignore_if_none(project_directory):
    """Add .projectignore if not found in project directory."""
    filename = os.path.join(project_directory, ".projectignore")
    gfilename = os.path.join(project_directory, ".gitignore")
    if not os.path.exists(filename) and not os.path.exists(gfilename):
        try:
            with codecs.open(filename, 'w', 'utf-8') as f:
                f.write(_default_projectignore)
        except IOError:
            pass



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
"""The ``unarchive`` command unpacks an archive of the project."""
from __future__ import absolute_import, print_function

from anaconda_project.internal.cli import console_utils
from anaconda_project.internal.cli.project_load import CliFrontend
import anaconda_project.project_ops as project_ops


def unarchive_command(archive_filename, project_dir):
    """Unpack an archive of the project.

    Returns:
        exit code
    """
    status = project_ops.unarchive(archive_filename, project_dir, frontend=CliFrontend())
    if status:
        print(status.status_description)
        return 0
    else:
        console_utils.print_status_errors(status)
        return 1


def main(args):
    """Start the unarchive command and return exit status code."""
    return unarchive_command(args.filename, args.directory)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
"""Control verbose output."""
from __future__ import absolute_import

import logging

_verbose_loggers = []


def push_verbose_logger(logger):
    """Push a logger to log verbose messgaes."""
    global _verbose_loggers
    _verbose_loggers.append(logger)


def pop_verbose_logger():
    """Remove the most recently-pushed verbose logger."""
    global _verbose_loggers
    assert len(_verbose_loggers) > 0
    _verbose_loggers.pop()


_cached_null_logger = None


def _null_logger():
    global _cached_null_logger
    if _cached_null_logger is None:
        logger = logging.getLogger(name='anaconda_project_null')
        logger.addHandler(logging.NullHandler())
        _cached_null_logger = logger
    return _cached_null_logger


def _verbose_logger():
    """Used internal to anaconda-project library to get the current verbose logger."""
    if len(_verbose_loggers) > 0:
        return _verbose_loggers[-1]
    else:
        return _null_logger()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
"""py2/py3 metaclass utility, from jinja2 via Bokeh."""


def with_metaclass(meta, *bases):
    """Add metaclasses in both Python 2 and Python 3.

    Function from jinja2/_compat.py. License: BSD.
    Use it like this::

        class BaseForm(object):
            pass

        class FormType(type):
            pass

        class Form(with_metaclass(FormType, BaseForm)):
            pass

    This requires a bit of explanation: the basic idea is to make a
    dummy metaclass for one level of class instantiation that replaces
    itself with the actual metaclass.  Because of internal type checks
    we also need to make sure that we downgrade the custom metaclass
    for one level to something closer to type (that's why __call__ and
    __init__ comes back from type etc.).

    This has the advantage over six.with_metaclass of not introducing
    dummy classes into the final MRO.
    """
    # this is a hack to keep yapf from inserting a blank line that pep257 complains about
    empty = {}

    class metaclass(meta):
        __call__ = type.__call__
        __init__ = type.__init__

        def __new__(cls, name, this_bases, d):
            if this_bases is None:
                return type.__new__(cls, name, (), d)
            return meta(name, bases, d)

    return metaclass('temporary_class', None, empty)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
from __future__ import absolute_import, print_function

from anaconda_project.internal.cli.main import _parse_args_and_run_subcommand
from anaconda_project.internal.simple_status import SimpleStatus


def test_unarchive_command(capsys, monkeypatch):
    def mock_unarchive(filename, project_dir, parent_dir=None, frontend=None):
        assert frontend is not None
        frontend.info("a")
        frontend.info("b")
        return SimpleStatus(success=True, description="DESC")

    monkeypatch.setattr('anaconda_project.project_ops.unarchive', mock_unarchive)
    code = _parse_args_and_run_subcommand(['anaconda-project', 'unarchive', 'foo.tar.gz', 'bar'])
    assert code == 0

    out, err = capsys.readouterr()
    assert 'a\nb\nDESC\n' == out
    assert '' == err


def test_unarchive_command_error(capsys, monkeypatch):
    def mock_unarchive(filename, project_dir, parent_dir=None, frontend=None):
        assert frontend is not None
        frontend.info("a")
        frontend.info("b")
        frontend.error("c")
        frontend.error("d")
        return SimpleStatus(success=False, description="DESC", errors=['c', 'd'])

    monkeypatch.setattr('anaconda_project.project_ops.unarchive', mock_unarchive)
    code = _parse_args_and_run_subcommand(['anaconda-project', 'unarchive', 'foo.tar.gz', 'bar'])
    assert code == 1

    out, err = capsys.readouterr()
    assert 'a\nb\n' == out
    assert 'c\nd\nDESC\n' == err



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
from __future__ import absolute_import, print_function

from anaconda_project.internal.simple_status import SimpleStatus


def test_simple_status_properties():
    good_status = SimpleStatus(success=True, description="quick brown fox", errors=["bar"])
    assert good_status
    assert good_status.status_description == "quick brown fox"
    assert good_status.errors == ["bar"]

    bad_status = SimpleStatus(success=False, description="quick brown fox", errors=["bar"])
    assert not bad_status
    assert bad_status.status_description == "quick brown fox"
    assert bad_status.errors == ["bar"]
